from flask import current_app
from invenio_base.utils import obj_or_import_string
from marshmallow.exceptions import ValidationError
from oarepo_requests.types.generic import NonDuplicableOARepoRequestType
from oarepo_requests.types.ref_types import ModelRefTypes
from oarepo_runtime.i18n import lazy_gettext as _
from typing_extensions import override
from oarepo_requests.utils import is_auto_approved, request_identity_matches
from ..actions.doi import CreateDoiAction, ValidateDataForDoiAction,  DeleteDoiAction
from oarepo_requests.actions.generic import OARepoSubmitAction

class DeleteDoiRequestType(NonDuplicableOARepoRequestType):
    type_id = "delete_doi"
    name = _("Cancel DOI registration")

    @classmethod
    @property
    def available_actions(cls):
        return {
            **super().available_actions,
            "accept": DeleteDoiAction,
            "submit": OARepoSubmitAction, #only accept?
        }

    description = _("Request for deletion of a registered DOI")
    receiver_can_be_none = True
    allowed_topic_ref_types = ModelRefTypes(published=False, draft=True)

    #do we have to check this again in create method?
    def is_applicable_to(self, identity, topic, *args, **kwargs):
        mapping_file = current_app.config.get("DATACITE_MAPPING")
        mapping = obj_or_import_string(mapping_file[topic.schema])()
        doi_value = mapping.get_doi_value(topic) #only make sense if there is registered doi
        #it is possible to cancel registration for only draft dois, which are associated only to record drafts.
        if doi_value and topic.is_draft and getattr(topic, "is_draft", False):
            return super().is_applicable_to(identity, topic, *args, **kwargs)
        else:
            return False

    @override
    def stateful_name(self, identity, *, topic, request=None, **kwargs):
        if is_auto_approved(self, identity=identity, topic=topic):
            return self.name
        if not request:
            return _("Request DOI cancellation")
        match request.status:
            case "submitted":
                return _("DOI cancellation requested")
            case _:
                return _("Request DOI cancellation")

    @override
    def stateful_description(self, identity, *, topic, request=None, **kwargs):
        if is_auto_approved(self, identity=identity, topic=topic):
            return _("Click to cancel DOI registration.")

        if not request:
            return _("Request permission to cancel DOI registration.")
        match request.status:
            case "submitted":
                if request_identity_matches(request.created_by, identity):
                    return _(
                        "Permission to cancel DOI registration requested. "
                        "You will be notified about the decision by email."
                    )
                if request_identity_matches(request.receiver, identity):
                    return _(
                        "You have been asked to approve the request to cancel DOI registration to a record. "
                        "You can approve or reject the request."
                    )
                return _("Permission to cancel DOI registration requested. ")
            case _:
                if request_identity_matches(request.created_by, identity):
                    return _("Submit request to get permission to cancel DOI registration.")

class AssignDoiRequestType(NonDuplicableOARepoRequestType):
    type_id = "assign_doi"
    name = _("Assign Doi")

    @classmethod
    @property
    def available_actions(cls):
        return {
            **super().available_actions,
            "accept": CreateDoiAction,
            "submit": ValidateDataForDoiAction,
        }

    description = _("Request for DOI assignment")
    receiver_can_be_none = True
    allowed_topic_ref_types = ModelRefTypes(published=True, draft=True)

    def can_create(self, identity, data, receiver, topic, creator, *args, **kwargs):
        mapping_file = current_app.config.get("DATACITE_MAPPING")
        mapping = obj_or_import_string(mapping_file[topic.schema])()
        errors = mapping.metadata_check(topic)
        if len(errors) > 0:
            raise ValidationError(
                message=errors
            )
        super().can_create(identity, data, receiver, topic, creator, *args, **kwargs)

    def is_applicable_to(self, identity, topic, *args, **kwargs):

        mode = current_app.config.get("DATACITE_MODE")
        if mode == "AUTOMATIC" or mode == "AUTOMATIC_DRAFT":
            return False
        mapping_file = current_app.config.get("DATACITE_MAPPING")
        mapping = obj_or_import_string(mapping_file[topic.schema])()
        doi_value = mapping.get_doi_value(topic) #if ANY doi already assigned, adding another is not possible
        if doi_value is not None:
            return False
        else:
            return super().is_applicable_to(identity, topic, *args, **kwargs)

    @override
    def stateful_name(self, identity, *, topic, request=None, **kwargs):
        if is_auto_approved(self, identity=identity, topic=topic):
            return self.name
        if not request:
            return _("Request DOI assignment")
        match request.status:
            case "submitted":
                return _("DOI assignment requested")
            case _:
                return _("Request DOI assignment")

    @override
    def stateful_description(self, identity, *, topic, request=None, **kwargs):
        if is_auto_approved(self, identity=identity, topic=topic):
            return _("Click to assign DOI.")

        if not request:
            return _("Request permission to assign DOI.")
        match request.status:
            case "submitted":
                if request_identity_matches(request.created_by, identity):
                    return _(
                        "Permission to assign DOI requested. "
                        "You will be notified about the decision by email."
                    )
                if request_identity_matches(request.receiver, identity):
                    return _(
                        "You have been asked to approve the request to assign DOI to a record. "
                        "You can approve or reject the request."
                    )
                return _("Permission to assign DOI requested. ")
            case _:
                if request_identity_matches(request.created_by, identity):
                    return _("Submit request to get permission to assign DOI.")


from flask import current_app
from invenio_base.utils import obj_or_import_string
from marshmallow.exceptions import ValidationError
from oarepo_requests.actions.generic import OARepoAcceptAction, OARepoSubmitAction

from oarepo_doi.api import community_slug_for_credentials, create_doi, delete_doi

class AssignDoiAction(OARepoAcceptAction):
    log_event = True

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.mode = current_app.config.get("DATACITE_MODE")
        self.url = current_app.config.get("DATACITE_URL")
        self.mapping = current_app.config.get("DATACITE_MAPPING")
        self.specified_doi = current_app.config.get("DATACITE_SPECIFIED_ID")

        self.username = None
        self.password = None
        self.prefix = None

    def credentials(self, community):
        if not community:
            credentials = current_app.config.get(
                "DATACITE_CREDENTIALS_DEFAULT"
            )
        else:
            credentials_def = current_app.config.get("DATACITE_CREDENTIALS")

            credentials = credentials_def.get(community, None)
            if not credentials:
                credentials = current_app.config.get(
                    "DATACITE_CREDENTIALS_DEFAULT"
                )

        self.username = credentials["username"]
        self.password = credentials["password"]
        self.prefix = credentials["prefix"]

class CreateDoiAction(AssignDoiAction):


    def execute(self, identity, uow, *args, **kwargs):
        topic = self.request.topic.resolve()
        slug = community_slug_for_credentials(topic.parent["communities"].get("default", None))

        self.credentials(slug)

        #todo - only public?
        if topic.is_draft:
            create_doi(self, topic, topic, None)
        else:
            create_doi(self, topic, topic, "publish")
        super().execute(identity, uow)

class DeleteDoiAction(AssignDoiAction):

    def execute(self, identity, uow, *args, **kwargs):
        topic = self.request.topic.resolve()
        slug = community_slug_for_credentials(topic.parent["communities"].get("default", None))

        self.credentials(slug)

        delete_doi(self, topic)

        super().execute(identity, uow)

class RegisterDoiAction(AssignDoiAction):

    def execute(self, identity, uow, *args, **kwargs):
        topic = self.request.topic.resolve()
        slug = community_slug_for_credentials(topic.parent["communities"].get("default", None))

        self.credentials(slug)

        create_doi(self, topic, topic, None)

        super().execute(identity, uow)
class ValidateDataForDoiAction(OARepoSubmitAction):
    log_event = True

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.mapping = current_app.config.get("DATACITE_MAPPING")

    def execute(self, identity, uow, *args, **kwargs):
        topic = self.request.topic.resolve()
        mapping = obj_or_import_string(self.mapping[topic.schema])()
        errors = mapping.metadata_check(topic)
        if len(errors) > 0:
            raise ValidationError(
                message=errors
            )

        super().execute(identity, uow)
